package App::pause;

our $DATE = '2015-03-25'; # DATE
our $VERSION = '0.14'; # VERSION

1;
# ABSTRACT: An API for PAUSE

__END__

=pod

=encoding UTF-8

=head1 NAME

App::pause - An API for PAUSE

=head1 VERSION

This document describes version 0.14 of App::pause (from Perl distribution WWW-PAUSE-Simple), released on 2015-03-25.

=head1 DESCRIPTION

This module is just an empty one to ease installation of this distribution (e.g.
C<< cpanm -n App::pause >>). The real implementation is in
L<WWW::PAUSE::Simple>.

=head1 SEE ALSO

L<WWW::PAUSE::Simple>

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/WWW-PAUSE-Simple>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-WWW-PAUSE-Simple>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=WWW-PAUSE-Simple>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
